"""Tests standard tap features using the built-in SDK tests library."""

import datetime

from singer_sdk.tap_base import Tap
from singer_sdk.streams.core import Stream
import tap_googleads.tap
import pytest
import json

import responses

SAMPLE_CONFIG = {
    "start_date": datetime.datetime.now(datetime.timezone.utc).strftime("%Y-%m-%d"),
    "client_id": "12345",
    "client_secret": "12345",
    "developer_token": "12345",
    "refresh_token": "12345",
    "customer_id": "12345",
    "login_customer_id": "12345",
}


@pytest.fixture
def mocked_responses():
    # Assert all reqests are fired is turned off because we have some issues with the Authenticator singleton.
    # Easiest way to explain the issue is to give steps to reproduce
    # Set assert_all_requests_are_fired to True
    # Run poetry run pytest (runs core tests, and test_customer_not_enabled
    # This job will fail
    # Run poetry run pytest test_customer_not_found.py
    # Test will pass
    # Ends up being due to the modules for the GoogleAdsTap not being reloaded, and how the authenticator metaclass we're using works
    with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
        yield rsps


def test_customer_not_enabled(mocked_responses):
    auth_response = {
        "access_token": "access_granted",
        "expires_in": "10000000",
    }

    customer_not_enabled_body = {
        "error": {
            "code": 403,
            "message": "The caller does not have permission",
            "status": "PERMISSION_DENIED",
            "details": [
                {
                    "@type": "type.googleapis.com/google.ads.googleads.v8.errors.GoogleAdsFailure",
                    "errors": [
                        {
                            "errorCode": {"authorizationError": "CUSTOMER_NOT_ENABLED"},
                            "message": "The customer can't be used because it isn't enabled.",
                        }
                    ],
                    "requestId": "y5qNuM6d1jVb6km9FEa1GG",
                }
            ],
        }
    }
    mocked_responses.add(
        responses.POST,
        # TODO cleanup long url, googleads.googleapis.com/* would suffice
        "https://googleads.googleapis.com/v14/customers/12345/googleAds:search?pageSize=10000&query=%0A%20%20%20%20%20%20%20%20%20%20%20%20SELECT%20campaign.id%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20,%20campaign.name%0A%20%20%20%20%20%20%20%20%20%20%20%20FROM%20campaign%20%0A%20%20%20%20%20%20%20%20%20%20%20%20ORDER%20BY%20campaign.id%0A%20%20%20%20%20%20%20%20",
        body=json.dumps(customer_not_enabled_body).encode("utf-8"),
        status=403,
        content_type="application/json",
    )
    mocked_responses.add(
        responses.POST,
        "https://www.googleapis.com/oauth2/v4/token?refresh_token=12345&client_id=12345&client_secret=12345&grant_type=refresh_token",  # TODO Clean up
        body=json.dumps(auth_response).encode("utf-8"),
        status=200,
        content_type="application/json",
    )
    # Don't need ENV variables here
    tap: Tap = tap_googleads.tap.TapGoogleAds(
        config=SAMPLE_CONFIG, parse_env_config=False
    )
    campaign_stream: Stream = tap.streams["campaign"]
    context = {"client_id": "12345"}
    campaign_stream.sync(context)



"""REST client handling, including GoogleAdsStream base class."""

from urllib.parse import urlencode, urljoin
from pathlib import Path
from typing import Any, Dict, Optional, Union, List, Iterable

from memoization import cached

from singer_sdk.helpers.jsonpath import extract_jsonpath
from singer_sdk.streams import RESTStream
from singer_sdk.exceptions import FatalAPIError, RetriableAPIError
from singer_sdk.pagination import JSONPathPaginator

from tap_googleads.auth import GoogleAdsAuthenticator
from tap_googleads.utils import replicate_pk_at_root


SCHEMAS_DIR = Path(__file__).parent / Path("./schemas")


class GoogleAdsStream(RESTStream):
    """GoogleAds stream class."""

    url_base = "https://googleads.googleapis.com/v14"

    records_jsonpath = "$[*]"  # Or override `parse_response`.
    next_page_token_jsonpath = "$.nextPageToken"
    primary_keys_jsonpaths = None
    _LOG_REQUEST_METRIC_URLS: bool = True

    @property
    @cached
    def authenticator(self) -> GoogleAdsAuthenticator:
        """Return a new authenticator object."""
        base_auth_url = "https://www.googleapis.com/oauth2/v4/token"
        auth_params = {
            "refresh_token": self.config["refresh_token"],
            "client_id": self.config["client_id"],
            "client_secret": self.config["client_secret"],
            "grant_type": "refresh_token",
        }
        auth_url = urljoin(base_auth_url, "?" + urlencode(auth_params))
        return GoogleAdsAuthenticator(stream=self, auth_endpoint=auth_url)

    @property
    def http_headers(self) -> dict:
        """Return the http headers needed."""
        headers = {}
        if "user_agent" in self.config:
            headers["User-Agent"] = self.config.get("user_agent")
        headers["developer-token"] = self.config["developer_token"]
        headers["login-customer-id"] = self.config["login_customer_id"]
        return headers

    def get_new_paginator(self) -> JSONPathPaginator:
        return JSONPathPaginator(self.next_page_token_jsonpath)

    def get_url_params(
        self, context: Optional[dict], next_page_token: Optional[Any]
    ) -> Dict[str, Any]:
        """Return a dictionary of values to be used in URL parameterization."""
        params: dict = {}
        if next_page_token:
            params["pageToken"] = next_page_token
        if self.replication_key:
            params["sort"] = "asc"
            params["order_by"] = self.replication_key
        return params

    def validate_response(self, response):
        # Still catch error status codes
        if response.status_code == 403:
            msg = (
                f"{response.status_code} Client Error: "
                f"{response.reason} for url: {response.url}"
            )
            data = response.json()
            details: dict = data.get("error").get("details")

            if (
                details
                and details[0]["errors"][0]["errorCode"]["authorizationError"]
                == "CUSTOMER_NOT_ENABLED"
            ):
                raise CustomerNotEnabledError(msg)

        if response.status_code == 429:
            msg = (
                f"{response.status_code} Client Error: "
                f"{response.reason} for path: {self.path}."
                f"response.json() {response.json()}:"
            )
            raise RetriableAPIError(msg)

        if 400 <= response.status_code < 500:
            msg = (
                f"{response.status_code} Client Error: "
                f"{response.reason} for path: {self.path}."
                f"response.json() {response.json()}:"
            )
            raise FatalAPIError(msg)

        elif 500 <= response.status_code < 600:
            msg = (
                f"{response.status_code} Server Error: "
                f"{response.reason} for path: {self.path}"
            )
            raise RetriableAPIError(msg)

    def get_records(self, context: Optional[dict]) -> Iterable[Dict[str, Any]]:
        """Return a generator of row-type dictionary objects.

        Each row emitted should be a dictionary of property names to their values.

        Args:
            context: Stream partition or context dictionary.

        Yields:
            One item per (possibly processed) record in the API.
        """
        try:
            for record in self.request_records(context):
                transformed_record = self.post_process(record, context)
                if transformed_record is None:
                    # Record filtered out during post_process()
                    continue
                yield transformed_record
        except CustomerNotEnabledError as e:
            self.logger.warning(
                "We hit the Customer Not Enabled error. "
                "Happens when we get a customer from the hierarchy list that "
                "isn't enabled anymore, most likely due to a customer being "
                f"disabled after the API that lists customers is called.  {e=}"
            )

    def post_process(self, row: dict, context: Optional[dict] = None) -> Optional[dict]:
        """As needed, append or transform raw data to match expected structure."""
        return replicate_pk_at_root(row, self.primary_keys_jsonpaths)


class CustomerNotEnabledError(Exception):
    """
    Customer Not Enabled, sometimes googles cache gives us customers that
    are not enabled.
    """



"""Tests standard tap features using the built-in SDK tests library."""

import datetime

from singer_sdk.testing import get_standard_tap_tests
from tap_googleads.tap import TapGoogleAds


SAMPLE_CONFIG = {
    "start_date": datetime.datetime.now(datetime.timezone.utc).strftime("%Y-%m-%d"),
    "client_id": "12345",
    "client_secret": "12345",
    "developer_token": "12345",
    "refresh_token": "12345",
    "customer_id": "12345",
    "login_customer_id": "12345",
}


# Run standard built-in tap tests from the SDK:
def test_standard_tap_tests():
    """Run standard tap tests from the SDK."""
    tests = get_standard_tap_tests(TapGoogleAds, config=SAMPLE_CONFIG)
    for test in tests:
        test()



"""GoogleAds tap class."""

from typing import List

from singer_sdk import Tap, Stream
from singer_sdk import typing as th  # JSON schema typing helpers
from datetime import date, timedelta

from tap_googleads.streams import (
    CampaignsStream,
    AdGroupsStream,
    AdGroupsPerformance,
    AccessibleCustomers,
    CustomerHierarchyStream,
    CampaignPerformance,
    CampaignPerformanceByAgeRangeAndDevice,
    CampaignPerformanceByGenderAndDevice,
    CampaignPerformanceByLocation,
    GeotargetsStream,
    ConversionsByLocation,
    CampaignLabel,
)

STREAM_TYPES = [
    CampaignsStream,
    AdGroupsStream,
    AdGroupsPerformance,
    AccessibleCustomers,
    CustomerHierarchyStream,
    CampaignPerformance,
    CampaignPerformanceByAgeRangeAndDevice,
    CampaignPerformanceByGenderAndDevice,
    CampaignPerformanceByLocation,
    GeotargetsStream,
    ConversionsByLocation,
    CampaignLabel,
]


class TapGoogleAds(Tap):
    """GoogleAds tap class."""

    name = "tap-googleads"

    # TODO: Add Descriptions
    config_jsonschema = th.PropertiesList(
        th.Property(
            "client_id",
            th.StringType,
            required=True,
            description="ClientID from Oauth Setup",
        ),
        th.Property(
            "client_secret",
            th.StringType,
            required=True,
            description="ClientSecret from Oauth Setup",
        ),
        th.Property(
            "developer_token",
            th.StringType,
            required=True,
            description="Developer Token from Google Ads Console",
        ),
        th.Property(
            "refresh_token",
            th.StringType,
            required=True,
            description="Refresh Token from Oauth dance",
        ),
        th.Property(
            "customer_id",
            th.StringType,
            required=True,
            description=(
                "Customer ID from Google Ads Console, note this should be the top "
                "level client. This tap will pull all subaccounts",
            ),
        ),
        th.Property(
            "login_customer_id",
            th.StringType,
            required=True,
            description=(
                "Customer ID that has access to the customer_id, note that they can be "
                "the same, but they don't have to be as this could be a Manager account"
            ),
        ),
        th.Property(
            "start_date",
            th.DateTimeType,
            default=(date.today() - timedelta(days=7)).strftime("%Y-%m-%dT%H:%M:%SZ"),
            required=True,
            description=(
                "Date to start our search from, applies to Streams where there is a "
                "filter date. Note that Google responds to Data in buckets of 1 Day "
                "increments"
            ),
        ),
        th.Property(
            "end_date",
            th.DateTimeType,
            default=date.today().strftime("%Y-%m-%dT%H:%M:%SZ"),
            required=True,
            description=(
                "Date to end our search on, applies to Streams where there is a filter "
                "date. Note that the query is BETWEEN start_date AND end_date"
            ),
        ),
    ).to_dict()

    def discover_streams(self) -> List[Stream]:
        """Return a list of discovered streams."""
        return [stream_class(tap=self) for stream_class in STREAM_TYPES]



"""Utility functions for tap functionality"""

from typing import Optional


def replicate_pk_at_root(data: dict, jsonpaths: Optional[list]) -> dict:
    """Replicates the indicated values as root properties.

    Arguments:
        data: The json object to be modified.
        jsonpaths: A list of json paths in format 'level1.level2.etc' that
            indicate which values are to be replicated as a root level property.

    Returns:
        A modified json record with the duplicate values as root level primary keys.
    """
    if not jsonpaths:
        return data

    pk_str = ""
    new_data = data.copy()
    for path in jsonpaths:
        levels = path.split(".")

        # recursively build pk value
        val = data.copy()
        for i, level in enumerate(levels):
            val = val[level]
            if i == len(levels) - 1:
                pk_str += ":" + str(val) if pk_str else str(val)

    new_data["_sdc_primary_key"] = pk_str

    return new_data



"""Stream type classes for tap-googleads."""

from pathlib import Path
from typing import Any, Dict, Optional, Iterable
from datetime import datetime

from singer_sdk import typing as th  # JSON Schema typing helpers

from tap_googleads.client import GoogleAdsStream

SCHEMAS_DIR = Path(__file__).parent / Path("./schemas")


class AccessibleCustomers(GoogleAdsStream):
    """Accessible Customers"""

    path = "/customers:listAccessibleCustomers"
    name = "accessible_customers"
    primary_keys = None
    replication_key = None
    # TODO add an assert for one record
    #    schema_filepath = SCHEMAS_DIR / "customer.json"
    schema = th.PropertiesList(
        th.Property("resourceNames", th.ArrayType(th.StringType))
    ).to_dict()

    def get_child_context(self, record: dict, context: Optional[dict]) -> dict:
        """Return a context dictionary for child streams."""
        return {"resourceNames": record["resourceNames"]}


class CustomerHierarchyStream(GoogleAdsStream):
    """
    Customer Hierarchy, inspiration from Google here
    https://developers.google.com/google-ads/api/docs/account-management/get-account-hierarchy.

    This stream is stictly to be the Parent Stream, to let all Child Streams
    know when to query the down stream apps.

    """

    # TODO add a seperate stream to get the Customer information and return i
    rest_method = "POST"
    path = "/customers/{client_id}/googleAds:search"
    records_jsonpath = "$.results[*]"
    name = "customer_hierarchy"
    primary_keys_jsonpaths = ["customerClient.id"]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    parent_stream_type = AccessibleCustomers
    schema_filepath = SCHEMAS_DIR / "customer_hierarchy.json"

    def get_url_params(
        self, context: Optional[dict], next_page_token: Optional[Any]
    ) -> Dict[str, Any]:
        params = super().get_url_params(context, next_page_token)
        params["pageSize"] = "10000"
        params["query"] = self.gaql
        return params

    @property
    def gaql(self):
        return """
            SELECT customer_client.client_customer
                 , customer_client.level
                 , customer_client.manager
                 , customer_client.descriptive_name
                 , customer_client.currency_code
                 , customer_client.time_zone
                 , customer_client.id
            FROM customer_client
            WHERE customer_client.level <= 1
            """

    # Goal of this stream is to send to children stream a dict of
    # login-customer-id:customer-id to query for all queries downstream
    def get_records(self, context: Optional[dict]) -> Iterable[Dict[str, Any]]:
        """Return a generator of row-type dictionary objects.

        Each row emitted should be a dictionary of property names to their values.

        Args:
            context: Stream partition or context dictionary.

        Yields:
            One item per (possibly processed) record in the API.
        """
        client_ids = []
        if self.config["login_customer_id"]:
            client_ids = [self.config["login_customer_id"]]
        else:
            # TODO when implementing this the headers need to be set properly
            client_ids = context["resourceNames"]

        for client in client_ids:
            client_id = client.split("/")[-1]
            context["client_id"] = client_id
            for row in self.request_records(context):
                row = self.post_process(row, context)
                # Don't search Manager accounts as we can't query them for everything
                if row["customerClient"]["manager"] is True:
                    continue
                yield row

    def get_child_context(self, record: dict, context: Optional[dict]) -> dict:
        """Return a context dictionary for child streams."""
        return {"client_id": record["customerClient"]["id"]}


class GeotargetsStream(GoogleAdsStream):
    """Geotargets, worldwide, constant across all customers"""

    rest_method = "POST"

    records_jsonpath = "$.results[*]"
    name = "geo_target_constant"
    primary_keys_jsonpaths = ["geoTargetConstant.resourceName"]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "geo_target_constant.json"
    parent_stream_type = None  # Override ReportsStream default as this is a constant
    path = "/customers/{login_customer_id}/googleAds:search"

    def get_url_params(
        self, context: Optional[dict], next_page_token: Optional[Any]
    ) -> Dict[str, Any]:
        params = super().get_url_params(context, next_page_token)
        params["pageSize"] = "10000"
        params["query"] = self.gaql
        return params

    gaql = """
        SELECT geo_target_constant.canonical_name
             , geo_target_constant.country_code
             , geo_target_constant.id
             , geo_target_constant.name
             , geo_target_constant.status
             , geo_target_constant.target_type 
        FROM geo_target_constant
    """


class ReportsStream(GoogleAdsStream):
    rest_method = "POST"
    parent_stream_type = CustomerHierarchyStream
    path = "/customers/{client_id}/googleAds:search"

    def get_url_params(
        self, context: Optional[dict], next_page_token: Optional[Any]
    ) -> Dict[str, Any]:
        params = super().get_url_params(context, next_page_token)
        params["pageSize"] = "10000"
        params["query"] = self.gaql
        return params

    @property
    def gaql(self):
        raise NotImplementedError

    @property
    def start_date(self):
        start_date = self.config["start_date"]
        format_date = datetime.strptime(start_date, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d"
        )
        return format_date

    @property
    def end_date(self):
        end_date = self.config["end_date"]
        format_date = datetime.strptime(end_date, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d"
        )
        return format_date

    @property
    def between_filter(self):
        return f"BETWEEN '{self.start_date}' AND '{self.end_date}'"


class CampaignsStream(ReportsStream):
    """Define custom stream."""

    @property
    def gaql(self):
        return """
            SELECT campaign.id
                 , campaign.name
            FROM campaign 
            ORDER BY campaign.id
        """

    records_jsonpath = "$.results[*]"
    name = "campaign"
    primary_keys_jsonpaths = ["campaign.resourceName"]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "campaign.json"


class AdGroupsStream(ReportsStream):
    """Define custom stream."""

    @property
    def gaql(self):
        return """
            SELECT ad_group.url_custom_parameters
                 , ad_group.type
                 , ad_group.tracking_url_template
                 , ad_group.targeting_setting.target_restrictions
                 , ad_group.target_roas
                 , ad_group.target_cpm_micros
                 , ad_group.status
                 , ad_group.target_cpa_micros
                 , ad_group.resource_name
                 , ad_group.percent_cpc_bid_micros
                 , ad_group.name
                 , ad_group.labels
                 , ad_group.id
                 , ad_group.final_url_suffix
                 , ad_group.excluded_parent_asset_field_types
                 , ad_group.effective_target_roas_source
                 , ad_group.effective_target_roas
                 , ad_group.effective_target_cpa_source
                 , ad_group.effective_target_cpa_micros
                 , ad_group.display_custom_bid_dimension
                 , ad_group.cpv_bid_micros
                 , ad_group.cpm_bid_micros
                 , ad_group.cpc_bid_micros
                 , ad_group.campaign
                 , ad_group.base_ad_group
                 , ad_group.ad_rotation_mode
            FROM ad_group 
        """

    records_jsonpath = "$.results[*]"
    name = "ad_group"
    primary_keys_jsonpaths = ["adGroup.resourceName"]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "ad_group.json"


class AdGroupsPerformance(ReportsStream):
    """AdGroups Performance"""

    @property
    def gaql(self) -> str:
        return f"""
        SELECT campaign.id
             , ad_group.id
             , metrics.impressions
             , metrics.clicks
             , metrics.cost_micros
             , segments.date
        FROM ad_group
        WHERE segments.date {self.between_filter}
        """

    records_jsonpath = "$.results[*]"
    name = "ad_group_performance"
    primary_keys_jsonpaths = ["campaign.resourceName", "adGroup.id", "segments.date"]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "ad_group_performance.json"


class CampaignPerformance(ReportsStream):
    """Campaign Performance"""

    @property
    def gaql(self) -> str:
        return f"""
            SELECT campaign.id
                 , campaign.name
                 , campaign.status
                 , campaign.start_date
                 , campaign.end_date
                 , campaign.advertising_channel_type
                 , campaign.advertising_channel_sub_type
                 , segments.device
                 , segments.date
                 , metrics.impressions
                 , metrics.clicks
                 , metrics.ctr
                 , metrics.average_cpc
                 , metrics.cost_micros 
            FROM campaign 
            WHERE segments.date {self.between_filter}
            """

    records_jsonpath = "$.results[*]"
    name = "campaign_performance"
    primary_keys_jsonpaths = ["campaign.resourceName", "segments.date"]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "campaign_performance.json"


class CampaignPerformanceByAgeRangeAndDevice(ReportsStream):
    """Campaign Performance By Age Range and Device"""

    @property
    def gaql(self) -> str:
        return f"""
            SELECT ad_group_criterion.age_range.type
                 , campaign.name
                 , campaign.id
                 , campaign.status
                 , ad_group.name
                 , ad_group.id
                 , segments.date
                 , segments.device
                 , ad_group_criterion.system_serving_status
                 , ad_group_criterion.bid_modifier
                 , metrics.clicks
                 , metrics.impressions
                 , metrics.ctr
                 , metrics.average_cpc
                 , metrics.cost_micros
                 , campaign.advertising_channel_type 
            FROM age_range_view 
            WHERE segments.date {self.between_filter}
           """

    records_jsonpath = "$.results[*]"
    name = "campaign_performance_by_age_range_and_device"
    primary_keys_jsonpaths = [
        "campaign.resourceName",
        "adGroup.id",
        "adGroupCriterion.ageRange.type",
        "segments.device",
        "segments.date",
    ]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "campaign_performance_by_age_range_and_device.json"


class CampaignPerformanceByGenderAndDevice(ReportsStream):
    """Campaign Performance By Age Range and Device"""

    @property
    def gaql(self) -> str:
        return f"""
            SELECT ad_group_criterion.gender.type
                 , campaign.name
                 , campaign.status
                 , campaign.id
                 , ad_group.name
                 , ad_group.id
                 , segments.date
                 , segments.device
                 , ad_group_criterion.system_serving_status
                 , ad_group_criterion.bid_modifier
                 , metrics.clicks
                 , metrics.impressions
                 , metrics.ctr
                 , metrics.average_cpc
                 , metrics.cost_micros
                 , campaign.advertising_channel_type 
            FROM gender_view 
            WHERE segments.date {self.between_filter}
        """

    records_jsonpath = "$.results[*]"
    name = "campaign_performance_by_gender_and_device"
    primary_keys_jsonpaths = [
        "campaign.resourceName",
        "adGroup.id",
        "adGroupCriterion.gender.type",
        "segments.device",
        "segments.date",
    ]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "campaign_performance_by_gender_and_device.json"


class CampaignPerformanceByLocation(ReportsStream):
    """Campaign Performance By Age Range and Device"""

    @property
    def gaql(self) -> str:
        return f"""
            SELECT campaign_criterion.location.geo_target_constant
                 , campaign.name
                 , campaign.id
                 , campaign_criterion.bid_modifier
                 , segments.date
                 , metrics.clicks
                 , metrics.impressions
                 , metrics.ctr
                 , metrics.average_cpc
                 , metrics.cost_micros 
            FROM location_view 
            WHERE segments.date {self.between_filter}
              AND campaign_criterion.status != 'REMOVED'
                """

    records_jsonpath = "$.results[*]"
    name = "campaign_performance_by_location"
    primary_keys_jsonpaths = [
        "campaign.resourceName",
        "locationView.resourceName",
        "segments.date",
    ]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "campaign_performance_by_location.json"


class ConversionsByLocation(ReportsStream):
    """Conversions By Location"""

    @property
    def gaql(self) -> str:
        return f"""
            SELECT campaign_criterion.location.geo_target_constant
                 , campaign.name
                 , campaign.id
                 , campaign_criterion.bid_modifier
                 , segments.date
                 , segments.conversion_action_category
                 , metrics.conversions 
            FROM location_view 
            WHERE segments.date {self.between_filter}
              AND campaign_criterion.status != 'REMOVED'
            """

    records_jsonpath = "$.results[*]"
    name = "conversion_by_location"
    primary_keys_jsonpaths = [
        "campaign.resourceName",
        "locationView.resourceName",
        "segments.date",
    ]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "conversion_by_location.json"


class CampaignLabel(ReportsStream):
    """Conversions By Location"""

    gaql = """
        SELECT campaign_label.campaign
             , campaign_label.label
             , campaign_label.resource_name
             , campaign.id
             , campaign.name
             , campaign.resource_name
             , campaign.status
             , customer.id
             , customer.resource_name
             , label.name
             , label.resource_name
             , label.id
             , label.status 
       FROM campaign_label
    """
    records_jsonpath = "$.results[*]"
    name = "campaign_label"
    primary_keys_jsonpaths = [
        "campaignLabel.resourceName",
    ]
    primary_keys = ["_sdc_primary_key"]
    replication_key = None
    schema_filepath = SCHEMAS_DIR / "campaign_label.json"



from tap_googleads.utils import replicate_pk_at_root


def test_replicate_pk_at_root_composite():
    d = {"foo": {"level": "1", "id": 10}, "bar": 15}
    pk = ["foo.id", "bar"]

    res = replicate_pk_at_root(d, pk)

    assert res == {
        "foo": {"level": "1", "id": 10},
        "_sdc_primary_key": "10:15",
        "bar": 15,
    }


def test_replicate_pk_at_root_simple():
    d = {"foo": {"level": "1", "id": 10}, "bar": 15}
    pk = ["bar"]

    res = replicate_pk_at_root(d, pk)

    assert res == {"foo": {"level": "1", "id": 10}, "_sdc_primary_key": "15", "bar": 15}


def test_replicate_pk_at_root_no_jsonpath():
    d = {"bar": 15}
    pk = None

    res = replicate_pk_at_root(d, pk)

    assert res == {"bar": 15}
